@@ -16,6 +16,7 @@
 ##	along with Fonty Python.  If not, see <http://www.gnu.org/licenses/>.
 
 import wx
+import wx.lib.stattext
 
 ## Setup wxPython to access translations : enables the stock buttons.
 langid = wx.LANGUAGE_DEFAULT # Picks this up from $LANG
@@ -75,7 +76,8 @@ class FontViewPanel(wx.Panel):
 	"""
 	def __init__(self, parent):
 		wx.Panel.__init__(self, parent, id = -1)
-		
+		self.firstrun = True
+
 		self.pageindex = 1 # I start here
 		self.total_number_of_pages = 0
 		
@@ -84,12 +86,18 @@ class FontViewPanel(wx.Panel):
 		self.TICKMAP = None
 		self.TICK = wx.Bitmap(fpsys.mythingsdir + "tick.png", type=wx.BITMAP_TYPE_PNG)
 		self.CROSS = wx.Bitmap(fpsys.mythingsdir + "cross.png", type=wx.BITMAP_TYPE_PNG)
-		
+	
+		#Sept 2009
+		self.SEGFAULT = wx.Bitmap(fpsys.mythingsdir + 'font_segfault.png', wx.BITMAP_TYPE_PNG)
+		self.NO_DRAW = wx.Bitmap(fpsys.mythingsdir + 'font_cannot_draw.png', wx.BITMAP_TYPE_PNG)
+		self.NOT_FOUND = wx.Bitmap(fpsys.mythingsdir + 'font_not_found.png', wx.BITMAP_TYPE_PNG)
+		self.INFO_ITEM = wx.Bitmap(fpsys.mythingsdir + 'font_info_item.png', wx.BITMAP_TYPE_PNG)
+		self.TICKSMALL = wx.Bitmap(fpsys.mythingsdir + "ticksmall.png", type=wx.BITMAP_TYPE_PNG)
+
 		## Main Label on top
 		sizerMainLabel = wx.BoxSizer(wx.HORIZONTAL) 
-		self.textMainInfo = wx.StaticText(self, -1, " ") 
-		self.textMainInfo.SetFont(wx.Font(10, wx.FONTFAMILY_DEFAULT, wx.NORMAL, wx.FONTWEIGHT_BOLD))
-		sizerMainLabel.Add(self.textMainInfo,1,wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_LEFT) 
+		self.textMainInfo = MyLabel(self)
+		sizerMainLabel.Add(self.textMainInfo,1,wx.ALIGN_LEFT)
 		
 		## Page choice and Filter controls
 		sizerOtherControls = wx.BoxSizer(wx.HORIZONTAL)
@@ -144,7 +152,7 @@ class FontViewPanel(wx.Panel):
 		## Now the sizer to hold all the fontview controls
 		self.sizerFontView = wx.BoxSizer( wx.VERTICAL )
 		## The Main label
-		self.sizerFontView.Add(sizerMainLabel, 0, wx.EXPAND | wx.TOP | wx.BOTTOM, border = 5 )
+		self.sizerFontView.Add(sizerMainLabel, 0, wx.EXPAND | wx.BOTTOM, border = 0 )
 		## The font view
 		self.sizerFontView.Add(self.scrolledFontView, 1, wx.EXPAND )
 
@@ -170,6 +178,12 @@ class FontViewPanel(wx.Panel):
 		ps.sub( update_font_view, self.MainFontViewUpdate ) ##DND: class FontViewPanel
 		ps.sub( reset_to_page_one, self.ResetToPageOne ) ##DND: class FontViewPanel 
 
+	def getWidthOfMiddle( self ):
+		if self.firstrun:
+			return fpsys.config.size[0]# Use the last known width of the entire window as an initial size estimate.
+		else:
+			return self.sizerFontView.GetSize()[0] # After this flag is tripped, the app is aware of sizes and we can get a real number.
+
 	def OnClearClick( self, event ):
 		self.inputFilter.SetValue("") #was .Clear(), but that does not work for a combo box.
 		self.filter = ""
@@ -236,7 +250,8 @@ class FontViewPanel(wx.Panel):
 			current_page = self.pageindex - 1
 			num_in_one_page = fpsys.config.numinpage
 			total_num_fonts = len(fpsys.state.filteredViewObject)
-
+			
+			## Many thanks to Michael Hoeft for this fix! I suck at math :)
 			# I miss the right words to explain this step, therefore an example:
 			# 	23 / 10 = 2
 			# 	23 % 10 = 3 > modulo > bool(3) = True = 1
@@ -248,18 +263,6 @@ class FontViewPanel(wx.Panel):
 			#	------------------------------------------
 			# 	4 + 0 = 4 >	4 pages
 			self.total_number_of_pages = (total_num_fonts / num_in_one_page) + bool(total_num_fonts % num_in_one_page)
-			#print "total_num_fonts=", total_num_fonts
-####		gross = total_num_fonts / float(num_in_one_page)
-####		
-####		if gross <= 1:
-####			## There are less than num_in_one_page fonts to be viewed at all.
-####			self.total_number_of_pages = 1
-####		else:
-####			## Okay, we have at least 1 page, perhaps more.
-####			whole_number_of_pages = int(gross)
-####			remainder = whole_number_of_pages % num_in_one_page
-####			if remainder > 0: whole_number_of_pages += 1
-####			self.total_number_of_pages = whole_number_of_pages
 
 			start = current_page * num_in_one_page #leaf thru the pages to the one we are on now.
 			fin = start + num_in_one_page
@@ -285,6 +288,7 @@ class FontViewPanel(wx.Panel):
 		self.scrolledFontView.CreateFitmaps( sublist ) # Tell my child to draw the fonts
 		self.EnableDisablePrevNext()
 
+		self.firstrun = False # After all the fitmaps are drawn, the sizer knows how wide it is, so we trip this flag (see getWidthOfMiddle)
 
 	def onMainClick(self, evt) :
 		"""
@@ -322,9 +326,9 @@ class FontViewPanel(wx.Panel):
 				if not bug:
 					ps.pub(print_to_status_bar,_("Selected fonts have been removed."))
 				else:
-					ps.pub(print_to_status_bar,_("There was an error writing the pog to disk. Nothing has been done"))
+					ps.pub(print_to_status_bar,_("There was an error writing the pog to disk. Nothing has been done."))
 		
-		## APPEND - Copy ttf to a pog.
+		## APPEND - Copy font to a pog.
 		if fpsys.state.action == "APPEND":
 			## We must append the fonts to the Pog
 			vo = fpsys.state.viewobject
@@ -450,7 +454,7 @@ class FontViewPanel(wx.Panel):
 		if Vpatt == "E": #NOTE : TESTING VPATT, not PATT - ergo: this covers E, EN, EP
 			## Empty "E" - when the chosen Folder or Pog has NO FONTS IN IT.
 			if Tpatt == "P":
-				lab = _("Your active Target Pog is:%s") % T.name
+				lab = _("Your active Target is %s") % T.name
 				status = _("Please choose a Source.")
 			else:
 				lab = _("There are no fonts in here.")
@@ -461,7 +465,7 @@ class FontViewPanel(wx.Panel):
 			
 		elif Patt == "FN":
 			#View a Folder, no target
-			lab = _("Viewing Folder:%s") % V.label()
+			lab = _("Viewing Folder %s") % V.label()
 			fpsys.state.cantick = False
 			btext = _("Nothing to do")
 			fpsys.state.action = "NOTHING_TO_DO" # We will test this in mainframe::OnMainClick
@@ -470,13 +474,13 @@ class FontViewPanel(wx.Panel):
 			#View a pog, no target
 			if V.isInstalled():
 				## Cannot remove fonts from an installed pog
-				lab = _("Viewing (installed) Pog: %s") % V.name
+				lab = _("Viewing (installed Pog)  %s") % V.name
 				btext = _("Nothing to do")
 				fpsys.state.action = "NOTHING_TO_DO"
 				fpsys.state.cantick = False
 				status = _("You cannot change an installed Pog.")
 			else:
-				lab = _("Viewing (editable) Pog: %s") % V.name
+				lab = _("Viewing (editable Pog)  %s") % V.name
 				fpsys.state.cantick = True
 				btext = _("Remove fonts from %s") % V.name
 				self.TICKMAP = self.CROSS
@@ -486,13 +490,13 @@ class FontViewPanel(wx.Panel):
 			#Folder to Pog
 			if T.isInstalled():
 				## We cannot put stuff into an installed pog
-				lab = _("Viewing Folder:%s") % V.label()
+				lab = _("Viewing Folder %s") % V.label()
 				btext = _("Nothing to do")
 				fpsys.state.action = "NOTHING_TO_DO"
 				fpsys.state.cantick = False
 				status = _("You cannot change an installed Pog.")
 			else:
-				lab = _("Append From: %(source)s To:%(target)s") % { "source":V.label(), "target":T.name }
+				lab = _("Append from %(source)s to %(target)s") % { "source":V.label(), "target":T.name }
 				btext = _("Put fonts into %s") % T.name
 				self.TICKMAP = self.TICK
 				fpsys.state.cantick = True
@@ -502,7 +506,7 @@ class FontViewPanel(wx.Panel):
 			#Pog to Pog
 			if T.isInstalled():
 				## We cannot put fonts into an installed pog
-				lab = _("Viewing Pog:%(source)s, but Pog:%(target)s is installed.") % {"source":V.name, "target":T.name}
+				lab = _("Viewing %(source)s, but Pog %(target)s is installed.") % {"source":V.name, "target":T.name}
 				btext = _("Nothing to do")
 				fpsys.state.action = "NOTHING_TO_DO"
 				fpsys.state.cantick = False
@@ -516,7 +520,7 @@ class FontViewPanel(wx.Panel):
 					fpsys.state.action = "NOTHING_TO_DO"
 					status = _("Your Source and Target are the same Pog.")
 				else: # Normal pog to pog
-					lab = _("Append from Pog:%(source)s into Pog:%(target)s") % {"source":V.name, "target":T.name}
+					lab = _("Append from %(source)s into %(target)s") % {"source":V.name, "target":T.name}
 					btext = _("Put fonts into %s") % T.name
 					self.TICKMAP = self.TICK
 					fpsys.state.cantick = True	 
@@ -532,9 +536,8 @@ class FontViewPanel(wx.Panel):
 			if not fpsys.state.viewobject.isInstalled():
 				ps.pub( toggle_purge_menu_item, True )
 		
-
 		self.buttMainLastLabel=btext
-		self.textMainInfo.SetLabel(lab)
+		self.textMainInfo.SetLabel( lab)
 		self.textMainInfo.Show()
 		if status is not "":
 			ps.pub(print_to_status_bar, status)
@@ -547,3 +550,60 @@ class FontViewPanel(wx.Panel):
 	def ResetToPageOne(self):
 		self.pageindex = 1 # I start here
 
+class MyLabel( wx.lib.stattext.GenStaticText ):
+	"""
+	To spice-up the info label I made this control. It draws a shape behind the text.
+	Thanks to Andrea: http://wiki.wxpython.org/CreatingCustomControls
+	"""
+	def __init__(self, parent):
+		self.FVP = parent
+		self.lab = u" " 
+		self.infoFont = wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.NORMAL, wx.FONTWEIGHT_BOLD)
+		self.light = wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DHIGHLIGHT )
+		self.dark = wx.SystemSettings.GetColour(wx.SYS_COLOUR_3DSHADOW)
+		self.back = parent.GetBackgroundColour()
+		self.h=100
+
+		self.VIEWICON = wx.Bitmap(fpsys.mythingsdir + "view16x16.png", type=wx.BITMAP_TYPE_PNG)
+
+		# call parent init after vital settings are done.
+		wx.lib.stattext.GenStaticText.__init__(self, parent, -1," ")
+		self.Bind(wx.EVT_PAINT, self.OnPaint)
+	def SetLabel( self, lab ):
+		self.lab=lab
+		self.Refresh()
+	def DoGetBestSize(self):
+		dc = wx.ClientDC(self)
+		dc.SetFont(self.infoFont)
+		bestw,besth = dc.GetTextExtent(self.lab) or (100,100)
+		besth += 8
+		best = wx.Size(bestw,besth)
+		self.CacheBestSize(best)
+		self.h=besth
+		return best
+	def OnPaint(self, event):
+		pdc = wx.PaintDC(self)
+		try:
+			dc = wx.GCDC(pdc)
+		except:
+			dc = pdc
+		w=(dc.GetFullTextExtent(self.lab,self.infoFont)[0] or 100) + 40 
+
+		#Now draw the thing:
+		rect = wx.Rect(0,0, w, self.h + 10)
+		
+		dc.SetPen(wx.Pen(self.dark,width=1))
+		dc.SetBrush(wx.Brush(self.back))
+
+		#rect.SetPosition( (0,0) )
+		dc.DrawRoundedRectangleRect(rect, 5)
+		
+		#The gradient under the text
+		dc.GradientFillLinear( wx.Rect(2, self.h-15, w-3, self.h-2), self.light, self.back, nDirection=wx.NORTH )
+
+		# The text
+		dc.SetFont(self.infoFont) 
+		dc.DrawText(self.lab, 27,5)
+
+		dc.DrawBitmap( self.VIEWICON, 6,5, True)
+
